(function() {
    var doc = document;
    var maxWidth = 0;
    var isBgColorChanged = false;

    var isLocalImage = function(nodeTarget) {
        return nodeTarget.className.indexOf("default-img") == -1;
    };

    var isActionLink = function(link) {
        return link.getAttribute("link-type") == "action";
    };

    var getErrorTips = function(link) {
        var c = link.children;
        var errTips = null;
        for (var i = 0, len = c.length; i < len; i++) {
            var child = c[i];
            if (child.className == "error-tips") {
                errTips = child;
            }
        }
        return errTips;
    };

    var removeErrorTips = function(nodeTarget) {
        var errTips = getErrorTips(nodeTarget);
        if (errTips) {
            errTips.parentNode.removeChild(errTips);
        }
    };

    var setText = function(target, text) {
        target.innerHTML = "";
        target.appendChild(doc.createTextNode(text));
    };

    function imageOnError(link) {
        var errTips = getErrorTips(link);
        if (!errTips) {
            errTips = doc.createElement("p");
            errTips.className = "error-tips";
            link.appendChild(errTips);
        }
        setText(errTips, "点击重新加载");
    }

    function replaceImage(nodeTarget, src) {
        if (isLocalImage(nodeTarget)) {
            return;
        }

        if (!src) {
            imageOnError(nodeTarget);
        } else {
            var img = new Image();

            if (maxWidth == 0) {
                maxWidth = nodeTarget.parentNode.style.width;
            }

            img.onload = function() {
                img.style.width = maxWidth;
                nodeTarget.innerHTML = "";
                nodeTarget.className = "";
                nodeTarget.appendChild(img);

                if (isBgColorChanged) {
                    isBgColorChanged = false;
                    doc.body.style.backgroundColor = "#ffffff";
                } else {
                    isBgColorChanged = true;
                    doc.body.style.backgroundColor = "#fffffa";
                }
            };
            img.onerror = function() {
                img.onload = null;
                imageOnError(img.parentNode);
                img.onerror = null;
            };
            img.src = src;
        }
    }

    function location(action, args) {
        args = args || {};
        window.location.href = "wumii://hybrid?info=" + encodeURIComponent(JSON.stringify({"action": action , "args": args}));
    }

    /*
    remove the commented out code when flash is ready.
    //check flash if loaded
    function checkFlashLoaded(videoNodes) {
        var video = videoNodes[1].children[0];
        var percent = Math.abs(video.PercentLoaded());
        if (percent == 100) {
            videoNodes[0].style.display = "none";
            video.style.height = "100%";
            video.style.visibility = "visible";
        } else {
            arguments.callee(videoNodes);
        }
    }

    var divs = article.getElementsByTagName("div");
    for (var i = 0, len = divs.length; i < len; i++) {
        if (divs[i].className == "wumii-video") {
            var children = divs[i].children;
            if (children[1]) {
                // has object or embed tag
                checkFlashLoaded(children);
            }
        }
    }*/

    var processImagesArr = [];

    function $$(selector) {
        var arr = [];
        if (!/\.| /.test(selector) && selector.substr(0, 1) == "#") {
            return doc.getElementById(selector.substring(1));
        } else {
            var el = doc.querySelectorAll(selector);
            for (var i = 0, len = el.length; i < len; i++) {
                arr.push(el[i]);
            }
        }
        return arr;
    }

    var likeCount = 0, commentCount = 0;
    var wrapper = $$("#wrapper");

    function replaceIconImage(src, node) {
        var img = new Image();
        img.onload = function() {
            var cls = node.className;
            if (cls) {
                img.className = cls;
            }
            node.parentNode.insertBefore(img, node);
            node.parentNode.removeChild(node);
        };
        img.src = src;
    }

    var styleTemp;
    function setFontSize(fontSize) {
        if (styleTemp) {
            styleTemp.parentNode.removeChild(styleTemp);
        }

        styleTemp = updateStyle(fontSize);
    }

    function replaceImagesToNightMode(nodes) {
        for (var i = 0, len = nodes.length; i < len; i++) {
            var node = nodes[i];
            var src = node.src;
            src = src.replace("images/", "images/night/");

            replaceIconImage(src, node);
        }
    }

    function updateStyle(fontSize, isNightMode, temp) {
        var style = doc.createElement("style");
        var styleStr = "#article, #article sup { font-size: " + fontSize + "px; line-height: " + fontSize * 23 / 16 + "px;}" +
                "#article h1, #article h2, #article h3 { font-size: " + (fontSize + 4) + "px;}" +
                "#article h4, #article h5, #article h6 { font-size: " + (fontSize + 2) + "px;}" +
                "#article p, #article div { text-indent: " + fontSize * 2+ "px;}" +
                "#article .wumii-image-desc { line-height: " + (fontSize + 4) + "px; font-size: " + (fontSize - 2) + "px;}" +
                "blockquote { margin-left: " + (fontSize - 4) / 2 + "px; padding-left: " + (fontSize - 4) / 2 + "px;}";

        if (isNightMode) {
            styleStr += "body, #wrapper { background-color: #282d32; color: #7e8c9a !important;} .wumii-image-desc, .options a { color: #5b656d !important;} #article .wumii-image .error-tips { color: #374046;}";
            if (temp != 2) {
                styleStr += "#header * { color: #7e8c9a !important;} #header .from-site { color: #5b656d !important;}";
                if (temp == 1) {
                    styleStr += "#header .from-site { border-left-color: #2e5371; background-color: #3c6b90; color: #d0dce7 !important;}";
                } else {
                    styleStr += "#header .title span { border-left-color: #2e5371;}";
                }

                replaceImagesToNightMode($$(".icon"));
                replaceImagesToNightMode($$("p.line img"));
            } else {
                styleStr += "#header .wumii-image { background-color: #282d32;}";
            }
        }

        style.innerHTML = styleStr;
        doc.head.appendChild(style);
        return style;
    }

    function registerLinkEvent(isNightMode) {
        var links = doc.getElementsByTagName("a");
        for (var i = 0, len = links.length; i < len; i++) {
            var link = links[i];
            if (isLocalImage(link)) {
                if (link.getElementsByTagName("img").length == 0 && !isActionLink(link)) {
                    link.addEventListener("click", function(e) {
                        e.preventDefault();
                        location("clickOnLink", {"url":this.getAttribute("href")});
                    });
                } else if (link.parentNode.className == "wumii-image") {
                    link.addEventListener("click", function(e) {
                        e.preventDefault(); 
                        location("showImageInGallery", {"url":this.getAttribute("href")});
                    });
                }
            } else {
                if (link.parentNode.className == "wumii-image") {
                    if (isNightMode) {
                        replaceIconImage("images/night/default.png", link.children[0]);
                    }
                    processImagesArr.push(link);
                    link.addEventListener("click", function(e) {
                        e.preventDefault(); 
                        if (!isLocalImage(this)) {
                            var errTips = getErrorTips(this);
                            if (errTips) {
                                setText(errTips, "正在加载...");
                            }
                            location("loadImage", {"url":this.getAttribute("href")});
                        } else {
                            location("showImageInGallery", {"url":this.getAttribute("href")});
                        }
                    });
                }
            }
        }
    }

    function updateHeaderStyle(temp) {
        var header = $$("#header"),
            imageNode = header.children[3];

        if (temp == 2) {
            if (imageNode) {
                var time = doc.createElement("span");
                time.className = "time";
                time.innerHTML = header.children[1].children[0].innerHTML;
                header.children[0].children[0].appendChild(time);

                var height = imageNode.children[0].children[0].height,
                    MAX_HEIGHT = 240;
                imageNode.style.top = "-" + (height - MAX_HEIGHT) / 2 + "px";

                var bgBar = doc.createElement("img");
                bgBar.className = "bg-bar";
                bgBar.src = "images/title_bg.png";
                $$("#header p.from-site")[0].appendChild(bgBar);

                var headerDetail = $$("#header div.like, #header div.comment");
                for (var i = 0, len = headerDetail.length; i < len; i++) {
                    var detail = headerDetail[i];

                    var imageNode = doc.createElement("img");
                    imageNode.className = "detail-bg";
                    imageNode.src = "images/detail_bg.png";

                    detail.appendChild(imageNode);

                    var icon = detail.children[1];
                    var imageSrc = icon.getAttribute("src");
                    imageSrc = imageSrc.replace(".png", "2.png");
                    icon.setAttribute("src", imageSrc);
                }

                $$("#header a")[0].addEventListener("click", function(e) {
                    e.preventDefault();
                });
            } else {
                temp = 1;
            }
        }

        var style = doc.createElement("link");
        style.href = "css/temp" + temp + ".css";
        style.rel = "stylesheet";
        style.type = "text/css";
        doc.head.appendChild(style);

        return temp;
    }

    function updateItemDetail(data) {
        var numObject = $$(".detail .num");
        for (var i = 0, len = numObject.length; i < len; i++) {
            var node = numObject[i];
            var num = data[node.parentNode.className];
            setText(node, num);
            var nextNode = node.nextSibling;
            if (nextNode && nextNode.nodeType != 1) {
                nextNode = nextNode.nextSibling;
            }

            if (nextNode && nextNode.className == "icon") {
                if (num == 0) {
                    node.parentNode.style.display = "none";
                } else {
                    node.parentNode.style.display = "block";
                }
            }
        }
    }

    function updateLikeState(isLike) {
        updateItemDetail({
            "like": isLike ? ++likeCount : --likeCount,
            "comment": commentCount
        });
    }

    function updateCommentsCount(commentsCount) {
        updateItemDetail({
            "like" : likeCount,
            "comment" : commentsCount
        });
    }

    function updateCountAndDetail(info) {
        likeCount = info.likeCount;
        commentCount = info.commentCount + info.weiboCommentCount;
        updateItemDetail({
            "like": likeCount,
            "comment": commentCount
        });
    }

    //replace remote images to local images
    function replaceImageSrc(remoteSrc, localSrc) {
        for (var i = 0, len = processImagesArr.length; i < len; i++) {
            var link = processImagesArr[i];
            if (link.getAttribute("href") == remoteSrc) {
                replaceImage(link, localSrc);
            }
        }
    }

    function setContent(contentInfo) {
        var range = doc.createRange();
        range.selectNodeContents(wrapper);
        var fragment = range.createContextualFragment(contentInfo.content);
        wrapper.appendChild(fragment);

        var temp = updateHeaderStyle(contentInfo.style);

        updateCountAndDetail(contentInfo);

        var isNightMode = contentInfo.isNightMode;

        registerLinkEvent(isNightMode);

        updateStyle(contentInfo.textSize, isNightMode, temp);

        $$("#likeItUsers").addEventListener("click", function() {
            location("clickOnUsersLikeIt");
        });

        location("setContentFinish");
    }

    function setArticleInfo(commentsInfo) {
        if (!commentsInfo.err) {
            updateCountAndDetail(commentsInfo);
        }
    }

    window["updateLikeState"] = updateLikeState;
    window["updateCommentsCount"] = updateCommentsCount;

    window["replaceImageSrc"] = replaceImageSrc;
    window["setContent"] = setContent;
    window["setArticleInfo"] = setArticleInfo;

    window["setFontSize"] = setFontSize;
})();